#ifndef POKEPLATINUM_UNK_02027B70_H
#define POKEPLATINUM_UNK_02027B70_H

#include "struct_decls/struct_02027854_decl.h"
#include "struct_defs/struct_02027BF4.h"

void sub_02027B70(UnkStruct_02027854 * param0);
void sub_02027B90(UnkStruct_02027854 * param0, int param1, const u16 * param2, int param3);
UnkStruct_02027BF4 * sub_02027BF4(int param0);
int sub_02027D04(const UnkStruct_02027854 * param0, int param1);
int sub_02027D10(const UnkStruct_02027854 * param0, int param1);
int sub_02027D18(const UnkStruct_02027854 * param0, int param1);
int sub_02027D34(const UnkStruct_02027854 * param0, int param1);
void sub_02027D40(UnkStruct_02027854 * param0, int param1, const UnkStruct_02027BF4 * param2, int param3);
void sub_02027D78(UnkStruct_02027854 * param0, int param1);
BOOL sub_02027D84(const UnkStruct_02027854 * param0, int param1);
void sub_02027D90(UnkStruct_02027854 * param0, int param1, BOOL param2);
int sub_02027D9C(const UnkStruct_02027854 * param0, int param1);
void sub_02027DA8(UnkStruct_02027854 * param0, int param1, int param2);
int sub_02027DB4(const UnkStruct_02027854 * param0, int param1);
int sub_02027DC0(UnkStruct_02027854 * param0, int param1);
void sub_02027EAC(UnkStruct_02027854 * param0, const UnkStruct_02027BF4 * param1, int param2);

#endif // POKEPLATINUM_UNK_02027B70_H
